@@ -32,9 +32,10 @@ sub disable {
 }
 
 sub _protect {
-  my $cb       = pop;
-  my $sigset   = POSIX::SigSet->new;
-  my $blockset = POSIX::SigSet->new(SIGCHLD);
+  shift if $_[0] && $_[0] eq $singleton;
+  my ($sig, $cb) = (@_ > 1 ? pop : SIGCHLD, pop);
+  my ($sigset, $blockset) = (POSIX::SigSet->new, POSIX::SigSet->new($sig));
+  $singleton->emit(protect => [$cb, $sig]);
   sigprocmask(SIG_BLOCK, $blockset, $sigset);
   my $r = $cb->();
   sigprocmask(SIG_SETMASK, $sigset);
@@ -68,7 +69,6 @@ sub collect {
   if ($singleton->resolve($pid)) {
     $singleton->_collect($pid => $status => $errno);
     $singleton->emit(collected => $singleton->resolve($pid));
-
   }
   else {
     $singleton->orphans->{$pid}
@@ -193,6 +193,7 @@ sub _prctl {
 
 *singleton = \&new;
 *session   = \&new;
+*protect   = \&_protect;
 
 1;
 
@@ -248,6 +249,16 @@ Emitted when we receive SIG_CHLD.
 
 Emitted when child process is collected and it's return status is available.
 
+=head2 protect
+
+    $session->on(protect => sub {
+      my ($self, $detail) = @_;
+      my ($cb, $signal) = @$detail;
+      ...
+    });
+
+Emitted when protected callbacks are fired.
+
 =head2 collected_orphan
 
     $session->on(collected_orphan => sub {
@@ -463,6 +474,17 @@ Unregister the corresponding L<Mojo::IOLoop::ReadWriteProcess> with the given pi
 
 Collect the status for the given pid.
 
+=head2 protect()
+
+    use Mojo::IOLoop::ReadWriteProcess::Session qw(session);
+    use POSIX;
+
+    my $return = session->protect(sub { print "Hello World\n" });
+
+    session->protect(sub { print "Hello World\n" } => SIGTERM);
+
+Try to protect the execution of the callback from signal interrupts.
+
 =head1 EXPORTS
 
 =head2 session()
